package performancemonster.loghandler;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;

public class LogSender
{
  private static final int DEFAULT_PORT = 4560;
  private static final int RECONNECTION_DELAY = 3000;
  private static final Logger logger = Logger.getLogger(LogSender.class);

  private InetAddress address;
  private int port = DEFAULT_PORT;
  private ObjectOutputStream oos;
  private Connector connector;

  /**
   * The <b>RemoteHost</b> option takes a string value which should be the host
   * name of the server where a
   * <code>link org.apache.log4j.net.SocketNode</code> is running.
   */
  public void setRemoteHost(String host)
  {
    logger.debug("setRemoteHost(" + host + ")");
    address = getAddressByName(host);
  }

  /**
   * The <b>Port</b> option takes a positive integer representing the port
   * where the server is waiting for connections.
   */
  public void setPort(int port)
  {
    logger.debug("setPort(" + port + ")");
    this.port = port;
  }

  public synchronized void sendLogMessage(LogMessage object)
  {
    if (logger.isDebugEnabled()) {
      logger.debug("sendLogMessage(" + object + ")");
    }
    if (address == null) {
      address = getAddressByName("localhost");
    }

    if (oos != null) {
      try {
        oos.writeObject(object);
        oos.flush();
        oos.reset();
      } catch (IOException e) {
        oos = null;
        logger.warn("Detected problem with connection", e);
        fireConnector();
      }
    }
  }

  /**
   * Drop the connection to the remote host and release the underlying connector
   * thread if it has been created
   */
  public void close()
  {
    logger.debug("close()");
    if (oos != null) {
      try {
        oos.close();
      } catch (IOException e) {
        logger.error("Could not close oos.", e);
      }
      oos = null;
    }
    if (connector != null) {
      connector.interrupted = true;
      connector = null; // allow gc
    }
  }

  public void connect()
  {
    logger.debug("connect()");
    if (this.address == null)
      return;
    try {
      // First, close the previous connection if any.
      close();
      oos = new ObjectOutputStream(new Socket(address, port).getOutputStream());
    } catch (IOException e) {
      String msg = "Could not connect to remote server at ["
          + address.getHostName() + "].";
      msg += " Will try again later.";
      fireConnector(); // fire the connector thread
      logger.error(msg, e);
    }
  }
  
  public String toString()
  {
    return "LogSender[" + address + ":" + port + "]";
  }

  private void fireConnector()
  {
    if (connector == null) {
      logger.debug("Starting a new connector thread.");
      connector = new Connector();
      connector.setDaemon(true);
      connector.setPriority(Thread.MIN_PRIORITY);
      connector.start();
    }
  }

  private InetAddress getAddressByName(String host)
  {
    try {
      return InetAddress.getByName(host);
    } catch (UnknownHostException e) {
      logger.error("Could not find address of [" + host + "].", e);
      return null;
    }
  }

  /**
   * The Connector will reconnect when the server becomes available again. It
   * does this by attempting to open a new connection every
   * <code>reconnectionDelay</code> milliseconds.
   * <p>
   * It stops trying whenever a connection is established. It will restart to
   * try reconnect to the server when previpously open connection is droppped.
   * 
   * @author Ceki G&uuml;lc&uuml;
   * @since 0.8.4
   */
  private class Connector extends Thread
  {
    boolean interrupted = false;

    public void run()
    {
      Socket socket;
      while (!interrupted) {
        try {
          sleep(RECONNECTION_DELAY);
          logger.debug("Attempting connection to " + address.getHostName());
          socket = new Socket(address, port);
          synchronized (this) {
            oos = new ObjectOutputStream(socket.getOutputStream());
            connector = null;
            logger.debug("Connection established. Exiting connector thread.");
            break;
          }
        } catch (InterruptedException e) {
          logger.debug("Connector interrupted. Leaving loop.");
          return;
        } catch (java.net.ConnectException e) {
          logger.debug("Remote host " + address.getHostName()
              + " refused connection.");
        } catch (IOException e) {
          logger.debug("Could not connect to " + address.getHostName()
              + ". Exception is " + e);
        }
      }
    }
  }
}
