// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.analyzer;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import performancemonster.analyzer.impl.AnalyzedGraphElement;
import performancemonster.analyzer.impl.AnalyzedLoggingEvent;
import performancemonster.graph.GraphModel;
import performancemonster.loghandler.LogListener;
import performancemonster.loghandler.LogMessage;

/**
 * Class that receives "beginning events" and "ending events" and updates a
 * given {@link GraphModel}.
 */
public class LogAnalyzer implements LogListener
{
  private static final Logger logger = Logger.getLogger(LogAnalyzer.class);

  // The stack keeps track of AnalyzedGraphElements with no ending event, so
  // that they can later be paired with an incoming ending event
  private final List<AnalyzedGraphElement> elementStack;

  // The GraphModel that will be updated "on the fly" as events are sent to
  // this LogAnalyzer
  private final GraphModel model;

  /**
   * Constructs a new LogAnalyzer which will update the given {@link GraphModel}
   * as events are sent to this LogAnalyzer.
   * 
   * @param model
   *          The {@link GraphModel} to update.
   */
  public LogAnalyzer(GraphModel model)
  {
    this.model = model;
    elementStack = new ArrayList<AnalyzedGraphElement>();
  }
  
  /**
   * Tells this objects that a new LoggingEvent has been received. Will update
   * the underlying {@link GraphModel}.
   */
  public void receiveLogMessage(LogMessage message)
  {
    AnalyzedLoggingEvent ale = new AnalyzedLoggingEvent(message);
    if (ale.getType() != AnalyzedLoggingEvent.TYPE_UNKNOWN) {
      analyzeLogs(ale);
    } else {
      logger.info("Ignoring " + message);
    }
  }

  /**
   * Updates the underlying {@link GraphModel} with the given event in the right
   * way, i.e. updates an existing entry in the {@link GraphModel} if the given
   * event is an "ending event", or creates a new entry if the given event is a
   * "beginning event".
   */
  private void analyzeLogs(AnalyzedLoggingEvent event)
  {
    if (event.getType() == AnalyzedLoggingEvent.TYPE_BEGIN) {
      logger.debug("Begin " + event.getMessage());
      AnalyzedGraphElement element = new AnalyzedGraphElement(event);
      model.putElement(element);
      elementStack.add(element);
    } else if (event.getType() == AnalyzedLoggingEvent.TYPE_END) {
      logger.debug("End " + event.getMessage());
      AnalyzedGraphElement element = findMatchingEvent(event);
      if (element != null) {
        AnalyzedGraphElement newElement = new AnalyzedGraphElement(element, event);
        model.removeElement(element);
        model.putElement(newElement);
      } else {
        logger.warn("No match for " + event);
      }
    } else {
      logger.info("Ignoring " + event);
    }
  }
  
  /**
   * Finds the AnalyzedGraphElement from the given list, which mathes the name
   * from "event" and is a beginning event. Removes the event from the given
   * list. Searches backwards, in order to simulate a stack.
   */
  private AnalyzedGraphElement findMatchingEvent(AnalyzedLoggingEvent event)
  {
    for (int i = elementStack.size() - 1; i >= 0; i--) {
      AnalyzedGraphElement element = elementStack.get(i);
      AnalyzedLoggingEvent foundEvent = element.getEvent();
      if (foundEvent != null
          && foundEvent.getMessage().equals(event.getMessage())
          && foundEvent.getThreadID() == event.getThreadID()) {
        elementStack.remove(i);
        return element;
      }
    }
    return null;
  }
}
