// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.analyzer.impl;

import performancemonster.graph.GraphElement;

/**
 * A GraphElement implementation used by the loghandler package. Immutable.
 */
public class AnalyzedGraphElement implements GraphElement
{
  private final long beginTime;
  private long endTime;
  private final String category;
  private final String message;
  private final String stackTrace;
  private final long threadID;
  AnalyzedLoggingEvent beginningEvent;
  AnalyzedLoggingEvent endEvent;

  /**
   * Creates a new AnalyzedGraphElement based on an AnalyzedLoggingEvent. The
   * "end" time will be set to System.currentTimeMillis().
   * 
   * @param event The wrapped AnalyzedLoggingEvent
   */
  public AnalyzedGraphElement(AnalyzedLoggingEvent event)
  {
    beginTime = event.getTimestamp();
    endTime = 0L;
    category = event.getCategory();
    message = event.getMessage();
    stackTrace = event.getStackTrace();
    threadID = event.getThreadID();
    this.beginningEvent = event;
  }
  
  /**
   * Creates a new {@link AnalyzedGraphElement} identical to the given object,
   * except that the end event is set.
   * 
   * @param other
   *          The other {@link AnalyzedGraphElement} to copy.
   * @param event
   *          The end event for the new {@link AnalyzedGraphElement}.
   */
  public AnalyzedGraphElement(AnalyzedGraphElement other, AnalyzedLoggingEvent event)
  {
    this.beginTime = other.beginTime;
    this.category = other.category;
    this.message = other.message;
    this.stackTrace = other.stackTrace;
    this.threadID = other.threadID;
    
    this.endTime = event.getTimestamp();
  }

  public String getCategory()
  {
    return category;
  }
  
  /**
   * Gives the message contained in the element.
   */
  public String getMessage()
  {
    return message;
  }
  
  /**
   * Gives the contained stack trace.
   */
  public String getStackTrace()
  {
    return stackTrace;
  }

  /**
   * Gives the beginning timestamp for this element.
   */
  public long getBeginning()
  {
    return beginTime;
  }

  /**
   * Gives the ending timestamp for this element.
   */
  public long getEnd()
  {
    if (endTime == 0L) {
      return System.currentTimeMillis();
    } else {
      return endTime;
    }
  }

  /**
   * Gives the wrapped AnalyzedLoggingEvent, or null if no AnalyzedLoggingEvent
   * is being wrapped.
   */
  public AnalyzedLoggingEvent getEvent()
  {
    return beginningEvent;
  }

  @Override public String toString()
  {
    return message;
  }
}