// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.analyzer.impl;

import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import performancemonster.loghandler.LogMessage;

/**
 * Wrapper for a LoggingEvent. Gets a timestamp and a type (unknown, begin, or
 * end).
 */
public class AnalyzedLoggingEvent
{
  public static final int TYPE_UNKNOWN = 0;
  public static final int TYPE_BEGIN = 1;
  public static final int TYPE_END = 2;

  private static final Logger logger = Logger
  .getLogger(AnalyzedLoggingEvent.class);

  private final long timestamp;
  private final LogMessage logMessage;
  private final String message;
  private final long threadID;
  private final int type;

  /**
   * Creates a new AnalyzedLoggingEvent based on the given LoggingEvent. The
   * type will be extracted from the message of the LoggingEvent, and the
   * timestamp will be taken from System.currentTimeMillis().
   */
  public AnalyzedLoggingEvent(LogMessage logMessage)
  {
    this.logMessage = logMessage;
    timestamp = logMessage.getTimestamp();
    threadID = logMessage.getThreadID();
    String messageFromLog = logMessage.getMessage();
    if (messageFromLog.startsWith("BEGIN ")) {
      logger.debug("BEGIN: " + messageFromLog);
      String messageBody = messageFromLog.substring(6);
      StringTokenizer st = new StringTokenizer(messageBody, "(");
      message = st.nextToken();
      logger.debug("BEGIN: " + message);
      type = TYPE_BEGIN;
    } else if (messageFromLog.startsWith("END ")) {
      logger.debug("END: " + messageFromLog);
      String messageBody = messageFromLog.substring(4);
      StringTokenizer st = new StringTokenizer(messageBody, "(");
      message = st.nextToken();
      logger.debug("END: " + message);
      type = TYPE_END;
    } else {
      message = "";
      type = TYPE_UNKNOWN;
    }
  }

  /**
   * Gives the timestamp when this object was created.
   */
  public long getTimestamp()
  {
    return timestamp;
  }

  public String getStackTrace()
  {
    return logMessage.getStackTrace();
  }

  @Override public String toString()
  {
    return "AnalyzedLoggingEvent[" + logMessage + "]";
  }

  /**
   * Gives the type (unknown, begin, end)
   */
  public int getType()
  {
    return type;
  }

  public String getCategory()
  {
    return logMessage.getCategory();
  }
  
  public String getMessage()
  {
    return message;
  }
  
  public long getThreadID()
  {
    return threadID;
  }
}