// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.graph.coloring;

import java.awt.Color;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Pattern;

import performancemonster.graph.GraphElement;
import performancemonster.graph.coloring.impl.ColorRuleChangeListener;

public class ColorRule
{
  private Color barColor = Color.RED;
  private Color borderColor = Color.BLACK;
  private String name;
  private String category = "";
  private String message = "";
  private String stackTrace = "";

  // Cached Pattern objects
  private Pattern categoryPattern;
  private Pattern messagePattern;
  private Pattern stackTracePattern;

  // Listeners
  private final Set<ColorRuleChangeListener> listeners;

  /**
   * Creates a new {@link ColorRule} with the specified name.
   * @param name Name of the new rule.
   */
  public ColorRule(String name)
  {
    this.name = name;
    this.listeners = new HashSet<ColorRuleChangeListener>();
  }
  
  /**
   * Creates a copy of this {@link ColorRule}, except for the listeners.
   * @return a copy of this {@link ColorRule}.
   */
  public ColorRule copy()
  {
    ColorRule newRule = new ColorRule(name);
    newRule.barColor = this.barColor;
    newRule.borderColor = this.borderColor;
    newRule.category = this.category;
    newRule.message = this.message;
    newRule.stackTrace = this.stackTrace;
    return newRule;
  }

  public boolean matches(GraphElement element)
  {
    if (!getCategoryPattern().matcher(element.getCategory()).find()) {
      return false;
    }
    if (!getMessagePattern().matcher(element.getMessage()).find()) {
      return false;
    }
    if (!getStackTracePattern().matcher(element.getStackTrace()).find()) {
      return false;
    }
    return true;
  }

  public Color getBarColor()
  {
    return barColor;
  }
  
  public void setBarColor(Color barColor)
  {
    if (barColor == null) {
      throw new IllegalArgumentException("Cannot set bar color to null");
    }
    this.barColor = barColor;
    notifyListeners();
  }

  public Color getBorderColor()
  {
    return borderColor;
  }

  public void setBorderColor(Color borderColor)
  {
    if (borderColor == null) {
      throw new IllegalArgumentException("Cannot set border color to null");
    }
    this.borderColor = borderColor;
    notifyListeners();
  }
  
  public String getName()
  {
    return name;
  }
  
  public void setName(String name)
  {
    this.name = name;
    notifyListeners();
  }

  public String getCategory()
  {
    return category;
  }
  
  public void setCategory(String category)
  {
    this.category = category;
    categoryPattern = null;
    notifyListeners();
  }
  
  public String getMessage()
  {
    return message;
  }
  
  public void setMessage(String message)
  {
    this.message = message;
    messagePattern = null;
    notifyListeners();
  }

  public String getStackTrace()
  {
    return stackTrace;
  }
  
  public void setStackTrace(String stackTrace)
  {
    this.stackTrace = stackTrace;
    stackTracePattern = null;
    notifyListeners();
  }
  
  public void addColorRuleChangeListener(ColorRuleChangeListener listener)
  {
    listeners.add(listener);
  }
  
  public void removeColorRuleChangeListener(ColorRuleChangeListener listener)
  {
    listeners.remove(listener);
  }
  
  private void notifyListeners()
  {
    for (ColorRuleChangeListener listener : listeners)
    {
      listener.colorRuleChanged();
    }
  }
  
  public String toString()
  {
    return "ColorRule[" + name + "," + barColor + "," + borderColor + "]";
  }
  
  private Pattern getCategoryPattern()
  {
    if (categoryPattern == null) {
      categoryPattern = Pattern.compile(category);
    }
    return categoryPattern;
  }

  private Pattern getMessagePattern()
  {
    if (messagePattern == null) {
      messagePattern = Pattern.compile(message);
    }
    return messagePattern;
  }
  
  private Pattern getStackTracePattern()
  {
    if (stackTracePattern == null) {
      stackTracePattern = Pattern.compile(stackTrace);
    }
    return stackTracePattern;
  }
}
