// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.instrumentation;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.*;
import java.util.*;

import org.apache.log4j.Logger;

import performancemonster.agent.AgentCommand;
import performancemonster.helpers.SetMap;
import performancemonster.instrumentation.exceptions.InstrumentationException;
import performancemonster.instrumentation.impl.*;
import performancemonster.instrumentation.impl.transform.Transform;
import performancemonster.instrumentation.settings.ClassSetting;
import performancemonster.instrumentation.settings.PackageSetting;
import performancemonster.instrumentation.settings.Setting;

/**
 * Facade class for all the instrumentation abilities of PerformanceMonster.
 */
public class InstrumentationFacade
{
  private static final Logger logger = Logger
      .getLogger(InstrumentationFacade.class);
  private String hostName;
  private int port;
  private Set<ClassDefinition> unsynchronizedClasses;
  private SetMap<ClassDefinition, MethodDefinition> instrumentedMethods;
  private final Map<String, PackageSetting> packageSettings;
  private final Map<String, ClassSetting> classSettings;
  private final Set<InstrumentationListener> listeners;

  public InstrumentationFacade()
  {
    logger.debug("constructor()");
    hostName = "localhost";
    port = 4000;
    unsynchronizedClasses = new HashSet<ClassDefinition>();
    instrumentedMethods = new SetMap<ClassDefinition, MethodDefinition>();
    packageSettings = new HashMap<String, PackageSetting>();
    classSettings = new HashMap<String, ClassSetting>();
    listeners = new HashSet<InstrumentationListener>();
  }

  public ClassSetting getClassSetting(String fullClassName)
  {
    logger.debug("getClassSetting(" + fullClassName + ")");
    return classSettings.get(fullClassName);
  }

  public PackageSetting getPackageSetting(String packageName)
  {
    logger.debug("getPackageSetting(" + packageName + ")");
    return packageSettings.get(packageName);
  }

  public void putClassSetting(ClassSetting classSetting)
  {
    logger.debug("putClassSetting(" + classSetting + ")");
    Setting previousSetting = getClassSetting(classSetting.getClassDefinition()
        .getFullName());
    if (previousSetting != null) {
      classSettings.remove(classSetting.getClassDefinition());
      notifySettingsRemoved(Collections.singleton(previousSetting));
    }
    classSettings.put(classSetting.getClassDefinition().getFullName(),
        classSetting);
    notifySettingsAdded(Collections.singleton((Setting) classSetting));
  }

  public void putPackageSetting(PackageSetting packageSetting)
  {
    logger.debug("putPackageSetting(" + packageSetting + ")");
    Setting previousSetting = getPackageSetting(packageSetting.getPackageName());
    if (previousSetting != null) {
      packageSettings.remove(packageSetting.getPackageName());
      notifySettingsRemoved(Collections.singleton(previousSetting));
    }
    packageSettings.put(packageSetting.getPackageName(), packageSetting);
    notifySettingsAdded(Collections.singleton((Setting) packageSetting));
  }

  public void removeSettings(Set<Setting> removedSettings)
  {
    logger.debug("removeSettings(" + removedSettings + ")");
    for (Setting setting : removedSettings) {
      if (setting instanceof ClassSetting) {
        classSettings.remove(((ClassSetting) setting).getClassDefinition());
      } else if (setting instanceof PackageSetting) {
        packageSettings.remove(((PackageSetting) setting).getPackageName());
      } else {
        throw new IllegalArgumentException("Unknown setting type: " + setting);
      }
    }
    notifySettingsRemoved(removedSettings);
  }

  public void addListener(InstrumentationListener listener)
  {
    logger.debug("addListener(" + listener + ")");
    listeners.add(listener);
  }

  public void removeListener(InstrumentationListener listener)
  {
    logger.debug("removeListener(" + listener + ")");
    listeners.remove(listener);
  }

  public void setHostName(String hostName)
  {
    logger.debug("setHostName(" + hostName + ")");
    this.hostName = hostName;
  }

  public void setPort(int port)
  {
    logger.debug("setPort(" + port + ")");
    this.port = port;
  }

  public String getHostName()
  {
    logger.debug("getHostName()");
    return hostName;
  }

  public int getPort()
  {
    logger.debug("getPort()");
    return port;
  }

  /**
   * Returns all found classes containing the given string in its name.
   * 
   * @param searchString
   *          String to search for.
   * @return All known classes containing <code>searchString</code>.
   */
  public Collection<ClassDefinition> findClasses(String searchString)
  {
    logger.debug("findClasses(" + searchString + ")");
    return MethodFinder.getInstance().doSearch(searchString);
  }

  /**
   * Gives all methods defined in a specified class.
   * 
   * @param classDefinition
   *          The specified class.
   * @return All methods defined in <code>classDefinition</code>.
   */
  public Collection<MethodDefinition> findMethods(
      ClassDefinition classDefinition)
  {
    logger.debug("findMethods(" + classDefinition + ")");
    return MethodFinder.getInstance().getMethods(classDefinition);
  }

  /**
   * Synchronizes the instrumentation information in PerformanceMonster with the
   * instrumented VM.
   */
  public void updateInstrumentation() throws InstrumentationException
  {
    logger.debug("updateInstrumentation()");
    // Run through all methods in the classpath, find out for each of them
    // whether or not they are to be instrumented
    for (ClassDefinition classDefinition : findClasses("")) {
      for (MethodDefinition methodDefinition : findMethods(classDefinition)) {
        Setting nearestSetting = getNearestSetting(methodDefinition);
        if (nearestSetting != null
            && nearestSetting.isInstrumentationEnabled(methodDefinition)) {
          addInstrumentedMethod(methodDefinition);
        } else {
          removeInstrumentedMethod(methodDefinition);
        }
      }
    }

    // Run through all unsynchronized classes and generate bytecode
    TransformCommand transformCommand = new TransformCommand();
    for (ClassDefinition classDefinition : unsynchronizedClasses) {
      Set<MethodDefinition> methodsToInstrument = instrumentedMethods
          .get(classDefinition);
      byte[] transformedBytecode;
      if (methodsToInstrument.isEmpty()) {
        transformedBytecode = classDefinition.getBytes();
      } else {
        transformedBytecode = Transform.transformMethods(methodsToInstrument);
      }
      transformCommand.addInstrumentation(classDefinition, transformedBytecode);
    }
    sendAgentCommand(transformCommand);
    initializeInstrumentation();

    // Now clear the "unsynchronizedClasses", since we have synchronized now
    unsynchronizedClasses.clear();
  }

  /**
   * Points the appender in the instrumented VM to PerformanceMonster.
   */
  public void initializeInstrumentation() throws InstrumentationException
  {
    InitializeCommand initializeCommand = new InitializeCommand();
    sendAgentCommand(initializeCommand);
  }

  private void addInstrumentedMethod(MethodDefinition methodDefinition)
  {
    logger.debug("addInstrumentedMethod(" + methodDefinition + ")");
    ClassDefinition classDefinition = methodDefinition.getClassDefinition();
    Set<MethodDefinition> methodsInstrumentedInClass = instrumentedMethods
        .get(classDefinition);
    if (!methodsInstrumentedInClass.contains(methodDefinition)) {
      unsynchronizedClasses.add(classDefinition);
      instrumentedMethods.put(classDefinition, methodDefinition);
    }
  }

  private void removeInstrumentedMethod(MethodDefinition methodDefinition)
  {
    logger.debug("removeInstrumentedMethod(" + methodDefinition + ")");
    ClassDefinition classDefinition = methodDefinition.getClassDefinition();
    Set<MethodDefinition> methodsInstrumentedInClass = instrumentedMethods
        .get(classDefinition);
    if (methodsInstrumentedInClass.contains(methodDefinition)) {
      unsynchronizedClasses.add(classDefinition);
      instrumentedMethods.removeValue(classDefinition, methodDefinition);
    }
  }

  private Setting getNearestSetting(MethodDefinition method)
  {
    String path = method.getClassDefinition().getFullName();
    if (classSettings.containsKey(path)) {
      return classSettings.get(path);
    }
    while (true) {
      int separator = path.lastIndexOf('.');
      if (separator == -1) {
        return null;
      } else {
        path = path.substring(0, separator);
      }
      if (packageSettings.containsKey(path)) {
        return packageSettings.get(path);
      }
    }
  }

  private void notifySettingsAdded(Set<Setting> addedSettings)
  {
    logger.debug("notifySettingsAdded(" + addedSettings + ")");
    for (InstrumentationListener listener : listeners) {
      listener.settingsAdded(addedSettings);
    }
  }

  private void notifySettingsRemoved(Set<Setting> removedSettings)
  {
    logger.debug("notifySettingsRemoved(" + removedSettings + ")");
    for (InstrumentationListener listener : listeners) {
      listener.settingsRemoved(removedSettings);
    }
  }

  private void sendAgentCommand(AgentCommand command)
      throws InstrumentationException
  {
    ObjectOutputStream oos = null;
    try {
      InetAddress address = InetAddress.getByName(hostName);
      oos = new ObjectOutputStream(new Socket(address, port).getOutputStream());
      oos.writeObject(command);
      oos.flush();
    } catch (UnknownHostException e) {
      throw new InstrumentationException("Problems sending agent command "
          + command + " to host " + hostName + ", port " + port, e);
    } catch (IOException e) {
      throw new InstrumentationException("Problems sending agent command "
          + command + " to host " + hostName + ", port " + port, e);
    } finally {
      if (oos != null) {
        try {
          oos.close();
        } catch (IOException e) {
          throw new InstrumentationException(
              "Problems closing ObjectOutputStream while sending agent command "
                  + command + " on host " + hostName + ", port " + port, e);
        }
      }
    }
  }
}
