// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.instrumentation.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.log4j.Logger;

import performancemonster.instrumentation.ClassDefinition;

public class DirectoryFileClassDefinition implements ClassDefinition
{
  private static final Logger logger = Logger.getLogger(ClassDefinition.class);

  private final String baseDirectory;
  private final String subDirectory;
  private final String fileName;
  private final String fullClassName;
  private final String shortClassName;
  private final String packageName;

  public static Collection<ClassDefinition> analyzeDirectory(
      String directoryName)
  {
    Collection<ClassDefinition> result = new ArrayList<ClassDefinition>();
    File directory = new File(directoryName);
    recurse(directory, "", result);
    return result;
  }
  
  private static void recurse(File base, String subdir,
      Collection<ClassDefinition> classes)
  {
    String directoryName = base.getAbsolutePath() + "/" + subdir;
    File directory = new File(directoryName);
    String[] children = directory.list();
    if (children == null) {
      return;
    }
    for (int i=0; i<children.length; i++) {
      String fileName = children[i];
      String fileNameWithPath = directoryName + "/" + fileName;
      File child = new File(fileNameWithPath);
      if (child.isFile()) {
        if (fileName.endsWith(".class")) {
          classes.add(new DirectoryFileClassDefinition(base.getAbsolutePath(),
              subdir, fileName));
        }
      } else if (child.isDirectory()) {
        String newSubDir = subdir + (subdir.length() > 0 ? "/" : "") + fileName;
        recurse(base, newSubDir, classes);
      } else {
        logger.warn("Could not determine type of file " + fileNameWithPath);
      }
    }
  }

  private DirectoryFileClassDefinition(String baseDirectory,
      String subDirectory, String fileName)
  {
    this.baseDirectory = baseDirectory;
    this.subDirectory = subDirectory;
    this.fileName = fileName;
    // fileName ends with .class, so we need to ignore the last 6 characters
    // when finding the class name
    fullClassName = (subDirectory + "/" + fileName.substring(0, fileName
        .length() - 6)).replace('/', '.');
    int separator = fullClassName.lastIndexOf('.');
    shortClassName = fullClassName.substring(separator + 1);
    packageName = (separator < 0) ? "" : fullClassName.substring(0, separator);
  }

  public String getFullName()
  {
    return fullClassName;
  }
  
  public String getShortName()
  {
    return shortClassName;
  }
  
  public String getPackageName()
  {
    return packageName;
  }

  public byte[] getBytes()
  {
    try {
      FileInputStream fileInputStream = new FileInputStream(baseDirectory + "/"
          + subDirectory + "/" + fileName);

      // Read 1024 bytes at a time, copy into a growing buffer.
      // TODO: This approach is sub-optimal, giving n squared complexity.
      byte[] result = new byte[0];
      byte[] buffer = new byte[1024];
      for (int readBytes = fileInputStream.read(buffer, 0, 1024); readBytes != -1; readBytes = fileInputStream
          .read(buffer, 0, 1024)) {
        byte[] newResult = new byte[result.length + readBytes];
        System.arraycopy(result, 0, newResult, 0, result.length);
        System.arraycopy(buffer, 0, newResult, result.length, readBytes);
        result = newResult;
      }
      return result;
    } catch (IOException e) {
      String errorString = "Could not read file " + fileName;
      logger.error(errorString);
      throw new IllegalStateException(errorString, e);
    }
  }

  public String toString()
  {
    return getFullName();
  }

  public int hashCode()
  {
    return fileName.hashCode();
  }

  public boolean equals(Object other)
  {
    if (this == other) {
      return true;
    }
    if (other == null) {
      return false;
    }
    if (!other.getClass().equals(getClass())) {
      return false;
    }

    DirectoryFileClassDefinition directoryFileClassDefinition = (DirectoryFileClassDefinition) other;
    return directoryFileClassDefinition.baseDirectory.equals(baseDirectory)
        && directoryFileClassDefinition.subDirectory.equals(subDirectory)
        && directoryFileClassDefinition.fileName.equals(fileName);
  }
}
