// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.instrumentation.impl;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import org.apache.log4j.Logger;

import performancemonster.instrumentation.ClassDefinition;

public class JarFileClassDefinition implements ClassDefinition
{
  private static final Logger logger = Logger
      .getLogger(JarFileClassDefinition.class);

  private final String jarFileName;
  private final String fileName;
  private final String fullClassName;
  private final String shortClassName;
  private final String packageName;

  public static Collection<ClassDefinition> analyzeJarFile(String jarFileName)
  {
    Collection<ClassDefinition> result = new ArrayList<ClassDefinition>();
    FileInputStream fileInputStream = null;
    JarInputStream jarInputStream = null;
    try {
      fileInputStream = new FileInputStream(jarFileName);
      jarInputStream = new JarInputStream(fileInputStream);

      for (JarEntry entry = jarInputStream.getNextJarEntry(); entry != null; entry = jarInputStream
          .getNextJarEntry()) {
        if (entry.getName().endsWith(".class")) {
          result.add(new JarFileClassDefinition(jarFileName, entry.getName()));
        }
      }
    } catch (IOException e) {
      logger.warn("Problems analyzing jar file " + jarFileName, e);
    } finally {
      if (fileInputStream != null) {
        try {
          fileInputStream.close();
        } catch (IOException e) {
          String errorString = "Error closing fileInputStream for file "
              + jarFileName;
          logger.error(errorString, e);
          throw new IllegalStateException(errorString, e);
        }
      }
      if (jarInputStream != null) {
        try {
          jarInputStream.close();
        } catch (IOException e) {
          String errorString = "Error closing jarInputStream for file "
              + jarFileName;
          logger.error(errorString, e);
          throw new IllegalStateException(errorString, e);
        }
      }
    }
    return result;
  }

  public JarFileClassDefinition(String jarFileName, String fileName)
  {
    this.jarFileName = jarFileName;
    this.fileName = fileName;
    if (fileName.endsWith(".class")) {
      fileName = fileName.substring(0, fileName.length() - 6);
    }
    fullClassName = fileName.replace('/', '.');
    int separator = fullClassName.lastIndexOf('.');
    shortClassName = fullClassName.substring(separator + 1);
    packageName = (separator < 0) ? "" : fullClassName.substring(0, separator);
  }

  public String getFullName()
  {
    return fullClassName;
  }
  
  public String getShortName()
  {
    return shortClassName;
  }
  
  public String getPackageName()
  {
    return packageName;
  }

  public byte[] getBytes()
  {
    try {
      FileInputStream fileInputStream = new FileInputStream(jarFileName);
      JarInputStream jarInputStream = new JarInputStream(fileInputStream);

      for (JarEntry entry = jarInputStream.getNextJarEntry(); entry != null; entry = jarInputStream
          .getNextJarEntry()) {
        if (entry.getName().equals(fileName)) {
          // Read 1024 bytes at a time, copy into a growing buffer.
          // TODO: This approach is sub-optimal, giving n squared complexity.
          byte[] result = new byte[0];
          byte[] buffer = new byte[1024];
          for (int readBytes = jarInputStream.read(buffer, 0, 1024); readBytes != -1; readBytes = jarInputStream
              .read(buffer, 0, 1024)) {
            byte[] newResult = new byte[result.length + readBytes];
            System.arraycopy(result, 0, newResult, 0, result.length);
            System.arraycopy(buffer, 0, newResult, result.length, readBytes);
            result = newResult;
          }
          return result;
        }
      }
    } catch (IOException e) {
      String errorString = "Could not read file " + fileName + " in jar file "
          + jarFileName;
      logger.error(errorString);
      throw new IllegalStateException(errorString, e);
    }
    String errorString = "Did not find entry " + fileName + " in jar file "
        + jarFileName;
    logger.error(errorString);
    throw new IllegalStateException(errorString);
  }

  public String toString()
  {
    return getFullName();
  }

  public int hashCode()
  {
    return jarFileName.hashCode() ^ fileName.hashCode() ^ fullClassName.hashCode();
  }

  public boolean equals(Object other)
  {
    if (this == other) {
      return true;
    }
    if (other == null) {
      return false;
    }
    if (!other.getClass().equals(getClass())) {
      return false;
    }

    JarFileClassDefinition jarFileClassDefinition = (JarFileClassDefinition) other;
    return jarFileClassDefinition.jarFileName.equals(jarFileName)
        && jarFileClassDefinition.fileName.equals(fileName)
        && jarFileClassDefinition.fullClassName.equals(fullClassName);
  }
}
