// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.loghandler;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * A daemon thread the processes connections from a
 * <code>org.apache.log4j.net.SocketAppender.html</code>.
 * 
 * @author <a href="mailto:oliver@puppycrawl.com">Oliver Burn</a>
 */
public class LogServer extends Thread
{
  private static final Logger logger = Logger.getLogger(LogServer.class);

  /**
   * Helper that actually processes a client connection. It receives events and
   * adds them to the supplied model.
   * 
   * @author <a href="mailto:oliver@puppycrawl.com">Oliver Burn</a>
   */
  private class Slurper implements Runnable
  {
    /** socket connection to read events from * */
    private final Socket mClient;

    /**
     * Creates a new <code>Slurper</code> instance.
     * 
     * @param aClient
     *          socket to receive events from
     */
    Slurper(Socket aClient)
    {
      mClient = aClient;
    }

    /** loops getting the events * */
    public void run()
    {
      logger.debug("Starting to get data");
      try {
        final ObjectInputStream ois = new ObjectInputStream(mClient
            .getInputStream());
        while (true) {
          LogMessage logMessage = (LogMessage) ois
              .readObject();
          notifyListeners(logMessage);
        }
      } catch (EOFException e) {
        logger.error("Error receiving data", e);
      } catch (SocketException e) {
        logger.error("Error receiving data", e);
      } catch (IOException e) {
        logger.error("Error receiving data", e);
      } catch (ClassNotFoundException e) {
        logger.error("Error receiving data", e);
      }

      try {
        mClient.close();
      } catch (IOException e) {
        logger.error("Error closing connection", e);
      }
    }
  }

  /** server for listening for connections * */
  private final ServerSocket mSvrSock;

  private List<LogListener> logListeners;

  /**
   * Creates a new <code>LoggingReceiver</code> instance.
   * 
   * @param aPort
   *          port to listen on
   * @throws IOException
   *           if an error occurs
   */
  public LogServer(int aPort) throws IOException
  {
    setDaemon(true);
    mSvrSock = new ServerSocket(aPort);
    logListeners = new ArrayList<LogListener>();
  }

  public void addLogListener(LogListener listener)
  {
    logListeners.add(listener);
  }

  /** Listens for client connections * */
  public void run()
  {
    try {
      while (true) {
        logger.debug("Waiting for a connection");
        final Socket client = mSvrSock.accept();
        logger.debug("Got a connection from "
            + client.getInetAddress().getHostName());
        final Thread t = new Thread(new Slurper(client));
        t.setDaemon(true);
        t.start();
      }
    } catch (IOException e) {
      // LOG.error("Error in accepting connections, stopping.", e);
    }
  }

  private void notifyListeners(LogMessage logMessage)
  {
    for (LogListener listener : logListeners) {
      listener.receiveLogMessage(logMessage);
    }
  }
}
