// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.colors;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SpringLayout;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;

import performancemonster.external.ColorExporter;
import performancemonster.external.ColorImporter;
import performancemonster.external.exceptions.ExternalException;
import performancemonster.graph.coloring.ColorChangeListener;
import performancemonster.graph.coloring.ColorRule;
import performancemonster.graph.coloring.Colorer;
import performancemonster.presentation.colors.impl.ColorAccessor;
import performancemonster.presentation.colors.impl.ColorRenderer;
import performancemonster.presentation.colors.impl.ColorRuleTableModel;
import performancemonster.presentation.helpers.Dialogs;
import performancemonster.presentation.helpers.LayoutManagerHandler;
import performancemonster.presentation.helpers.SpringUtilities;

public class ColorGUI
{
  private static final Logger logger = Logger.getLogger(ColorGUI.class);

  private final Colorer colorer;
  private final Colorer colorerBackup;
  private final JFrame frame;
  private JFileChooser fileChooser;

  // All listeners registered in the colorer must be put in this set, so that
  // they can be collectively unregistered when the window is shut down
  private final Set<ColorChangeListener> colorChangeListeners;

  private int colorRuleNumber = 0;
  private volatile ColorRule selectedColorRule = null;
  private File lastSavedFile;

  private JTextField nameTextField;
  private JButton barColorButton;
  private JButton borderColorButton;
  private JTextField categoryTextField;
  private JTextField messageTextField;
  private JTextField stackTraceTextField;
  private Component ruleDetailEditor;
  private JTable table;

  public ColorGUI(Colorer colorer)
  {
    this.colorer = colorer;
    this.colorerBackup = colorer.copy();
    this.colorChangeListeners = new HashSet<ColorChangeListener>();

    frame = new JFrame("Coloring");
    frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    frame.addWindowListener(new WindowAdapter()
    {
      @Override
      public void windowClosing(WindowEvent e)
      {
        cancel();
      }
    });
    
    LayoutManagerHandler buttonLayouter = new LayoutManagerHandler(
        LayoutManagerHandler.Direction.VERTICAL);
    buttonLayouter.addComponent(createDefaultColorPanel(), false, 0.0);
    buttonLayouter.addComponent(new JSeparator(), false, 0.0);
    buttonLayouter.addComponent(createRuleListPanel(), true, 1.0);
    buttonLayouter.addComponent(new JSeparator(), false, 0.0);
    buttonLayouter.addComponent(createOkCancelPanel(), false, 0.0);

    frame.add(buttonLayouter.getPanel());
    createMenus();
    frame.pack();
  }

  public void show()
  {
    frame.setVisible(true);
  }

  public JFrame getFrame()
  {
    return frame;
  }

  private Component createDefaultColorPanel()
  {
    // Labels
    JLabel backgroundLabel = new JLabel("Background:", JLabel.TRAILING);
    JLabel defaultBarLabel = new JLabel("Default bar:", JLabel.TRAILING);
    JLabel defaultBorderLabel = new JLabel("Default border:", JLabel.TRAILING);

    // Color buttons
    JButton backgroundButton = createColorButton("Choose background color",
        new ColorAccessor()
        {
          public Color getColor()
          {
            return colorer.getBackgroundColor();
          }

          public void setColor(Color color)
          {
            colorer.setBackgroundColor(color);
          }
        });
    JButton defaultBarButton = createColorButton("Choose default bar color",
        new ColorAccessor()
        {
          public Color getColor()
          {
            return colorer.getDefaultBarColor();
          }

          public void setColor(Color color)
          {
            colorer.setDefaultBarColor(color);
          }
        });
    JButton defaultBorderButton = createColorButton(
        "Choose default border color", new ColorAccessor()
        {
          public Color getColor()
          {
            return colorer.getDefaultBorderColor();
          }

          public void setColor(Color color)
          {
            colorer.setDefaultBorderColor(color);
          }
        });

    // Attach labels to color buttons
    backgroundLabel.setLabelFor(backgroundButton);
    defaultBarLabel.setLabelFor(defaultBarButton);
    defaultBorderLabel.setLabelFor(defaultBorderButton);

    JPanel topColors = new JPanel(new SpringLayout());

    // Add all labels and buttons to the panel
    topColors.add(backgroundLabel);
    topColors.add(backgroundButton);
    topColors.add(defaultBarLabel);
    topColors.add(defaultBarButton);
    topColors.add(defaultBorderLabel);
    topColors.add(defaultBorderButton);

    // Arrange the grid
    SpringUtilities.makeCompactGrid(topColors, // panel
        3, 2, // rows, cols
        6, 6, // initX, initY
        6, 6); // xPad, yPad

    return topColors;
  }

  private void unregisterListeners()
  {
    for (ColorChangeListener listener : colorChangeListeners) {
      colorer.removeChangeListener(listener);
    }
  }
  
  private void close()
  {
    frame.setVisible(false);
    frame.dispose();
  }

  private void createMenus()
  {
    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic(KeyEvent.VK_F);
    menuBar.add(fileMenu);
    
    // File->Load Ctrl-L
    JMenuItem openItem = new JMenuItem("Open", KeyEvent.VK_O);
    openItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_O, ActionEvent.CTRL_MASK));
    openItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        open();
      }
    });
    fileMenu.add(openItem);

    // File->Save Ctrl-S
    JMenuItem saveItem = new JMenuItem("Save", KeyEvent.VK_S);
    saveItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    saveItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        save();
      }
    });
    fileMenu.add(saveItem);
    
    // File->Save As...
    JMenuItem saveAsItem = new JMenuItem("Save as...");
    saveAsItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        saveAs();
      }
    });
    fileMenu.add(saveAsItem);

    frame.setJMenuBar(menuBar);
  }

  private void saveToFile(File file)
  {
    try {
      ColorExporter exporter = new ColorExporter(colorer);
      exporter.exportToFile(file);
    } catch (ExternalException e) {
      String errorString = "Could not save to file " + file;
      logger.error(errorString, e);
      Dialogs.getInstance().showError(frame, errorString, e);
    }
  }

  private void handleTableChange()
  {
    if (!colorer.contains(selectedColorRule)) {
      selectedColorRule = null;
    }

    if (selectedColorRule == null) {
      // Clear the contents of the components
      nameTextField.setText("");
      barColorButton.setBackground(Color.RED);
      borderColorButton.setBackground(Color.BLACK);
      categoryTextField.setText("");
      messageTextField.setText("");
      stackTraceTextField.setText("");

      // Disable the components
      nameTextField.setEnabled(false);
      barColorButton.setEnabled(false);
      borderColorButton.setEnabled(false);
      categoryTextField.setEnabled(false);
      messageTextField.setEnabled(false);
      stackTraceTextField.setEnabled(false);

      table.clearSelection();
    } else {
      // Set the contents of the components
      nameTextField.setText(selectedColorRule.getName());
      barColorButton.setBackground(selectedColorRule.getBarColor());
      borderColorButton.setBackground(selectedColorRule.getBorderColor());
      categoryTextField.setText(selectedColorRule.getCategory());
      messageTextField.setText(selectedColorRule.getMessage());
      stackTraceTextField.setText(selectedColorRule.getStackTrace());

      // Enable the components
      nameTextField.setEnabled(true);
      barColorButton.setEnabled(true);
      borderColorButton.setEnabled(true);
      categoryTextField.setEnabled(true);
      messageTextField.setEnabled(true);
      stackTraceTextField.setEnabled(true);

      int position = colorer.getColorRules().indexOf(selectedColorRule);
      table.getSelectionModel().setSelectionInterval(position, position);
    }
  }
  
  private Component createRuleListPanel()
  {
    table = new JTable(new ColorRuleTableModel(colorer));
    JScrollPane scrollPane = new JScrollPane(table);

    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    // Set up renderer for colors
    table.setDefaultRenderer(Color.class, new ColorRenderer(true));

    JPanel panel = new JPanel(new SpringLayout());
    ruleDetailEditor = createRuleDetailEditor();
    panel.add(scrollPane);
    panel.add(createNewDeleteMoveupMovedownPanel());
    panel.add(ruleDetailEditor);

    SpringUtilities.makeCompactGrid(panel, // panel
        3, 1, // rows, cols
        6, 6, // initX, initY
        6, 6); // xPad, yPad

    table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e)
      {
        int selectedRow = table.getSelectedRow();
        if (selectedRow != -1) {
          selectedColorRule = colorer.getColorRule(selectedRow);
        }
        handleTableChange();
      }});
    
    // Create a listener on the colorer, so that we can track all changes in
    // the underlying model
    colorer.addChangeListener(new ColorChangeListener()
    {
      public void coloringChanged()
      {
        handleTableChange();
      }
    });
    
    handleTableChange();
    return panel;
  }

  private Component createNewDeleteMoveupMovedownPanel()
  {
    JButton newButton = new JButton("New");
    JButton deleteButton = new JButton("Delete");
    JButton moveUpButton = new JButton("Move up");
    JButton moveDownButton = new JButton("Move down");

    newButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ColorRule newRule = new ColorRule("rule" + colorRuleNumber++);
        selectedColorRule = newRule;
        colorer.appendColorRule(newRule);
      }
    });
    deleteButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ColorRule oldSelectedColorRule = selectedColorRule;
        selectedColorRule = null;
        colorer.removeColorRule(oldSelectedColorRule);
      }
    });
    moveUpButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        colorer.moveUp(selectedColorRule);
      }
    });
    moveDownButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        colorer.moveDown(selectedColorRule);
      }
    });

    JPanel panel = new JPanel(new SpringLayout());
    panel.add(newButton);
    panel.add(deleteButton);
    panel.add(moveUpButton);
    panel.add(moveDownButton);

    SpringUtilities.makeCompactGrid(panel, // panel
        1, 4, // rows, cols
        6, 6, // initX, initY
        6, 6); // xPad, yPad
    return panel;
  }

  private Component createRuleDetailEditor()
  {
    JPanel panel = new JPanel(new SpringLayout());
    nameTextField = new JTextField();
    barColorButton = createColorButton("Choose bar color", new ColorAccessor()
    {
      public Color getColor()
      {
        if (selectedColorRule != null) {
          return selectedColorRule.getBarColor();
        }
        return Color.RED;
      }

      public void setColor(Color color)
      {
        selectedColorRule.setBarColor(color);
      }
    });
    borderColorButton = createColorButton("Choose border color",
        new ColorAccessor()
        {
          public Color getColor()
          {
            if (selectedColorRule != null) {
              return selectedColorRule.getBorderColor();
            }
            return Color.BLACK;
          }

          public void setColor(Color color)
          {
            selectedColorRule.setBorderColor(color);
          }
        });
    categoryTextField = new JTextField();
    messageTextField = new JTextField();
    stackTraceTextField = new JTextField();

    nameTextField.addFocusListener(new FocusAdapter()
    {
      public void focusLost(FocusEvent e)
      {
        selectedColorRule.setName(nameTextField.getText());
      }
    });
    categoryTextField.addFocusListener(new FocusAdapter()
    {
      public void focusLost(FocusEvent e)
      {
        selectedColorRule.setCategory(categoryTextField.getText());
      }
    });
    messageTextField.addFocusListener(new FocusAdapter()
    {
      public void focusLost(FocusEvent e)
      {
        selectedColorRule.setMessage(messageTextField.getText());
      }
    });
    stackTraceTextField.addFocusListener(new FocusAdapter()
    {
      public void focusLost(FocusEvent e)
      {
        selectedColorRule.setStackTrace(stackTraceTextField.getText());
      }
    });

    panel.add(new JLabel("Name:"));
    panel.add(nameTextField);
    panel.add(new JLabel("Bar:"));
    panel.add(barColorButton);
    panel.add(new JLabel("Border:"));
    panel.add(borderColorButton);
    panel.add(new JLabel("Category:"));
    panel.add(categoryTextField);
    panel.add(new JLabel("Message:"));
    panel.add(messageTextField);
    panel.add(new JLabel("Stack trace:"));
    panel.add(stackTraceTextField);

    SpringUtilities.makeCompactGrid(panel, // panel
        6, 2, // rows, cols
        6, 6, // initX, initY
        6, 6); // xPad, yPad

    return panel;
  }

  private Component createOkCancelPanel()
  {
    JButton okButton = new JButton("OK");
    JButton cancelButton = new JButton("Cancel");

    okButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ok();
      }
    });
    cancelButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        cancel();
      }
    });

    JPanel panel = new JPanel(new GridLayout(1, 2));
    panel.add(okButton);
    panel.add(cancelButton);

    return panel;
  }

  private JButton createColorButton(final String titleText,
      final ColorAccessor colorAccessor)
  {
    final JButton colorButton = new JButton("");
    colorButton.setBackground(colorAccessor.getColor());
    colorButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        Color c = JColorChooser.showDialog(colorButton, titleText, colorButton
            .getBackground());
        if (c != null) {
          colorAccessor.setColor(c);
        }
      }
    });
    ColorChangeListener listener = new ColorChangeListener()
    {
      public void coloringChanged()
      {
        colorButton.setBackground(colorAccessor.getColor());
        colorButton.setForeground(colorAccessor.getColor());
      }
    };
    colorer.addChangeListener(listener);
    colorChangeListeners.add(listener);
    return colorButton;
  }

  private JFileChooser getFileChooser()
  {
    if (fileChooser == null) {
      fileChooser = new JFileChooser();
    }
    return fileChooser;
  }
  
  private void rollback()
  {
    colorer.replaceWith(colorerBackup);
  }

  /**
   * Called when the user chooses "Open" from the pulldown menu.
   */
  private void open()
  {
    if (getFileChooser().showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
      File fileToOpen = getFileChooser().getSelectedFile();
      try {
        ColorImporter importer = new ColorImporter(colorer);
        importer.importFromFile(fileToOpen);
        lastSavedFile = fileToOpen;
      } catch (ExternalException e) {
        String errorString = "Could not open file " + fileToOpen;
        logger.error(errorString, e);
        Dialogs.getInstance().showError(frame, errorString, e);
      }
    }
  }

  /**
   * Called when the user chooses "Save" from the pulldown menu.
   */
  private void save()
  {
    if (lastSavedFile == null) {
      saveAs();
    } else {
      saveToFile(lastSavedFile);
    }
  }
  
  /**
   * Called when the user chooses "Save as..." from the pulldown menu.
   */
  private void saveAs()
  {
    if (getFileChooser().showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
      File chosenFile = getFileChooser().getSelectedFile();
      if (chosenFile.exists()
          && !Dialogs.getInstance().showYesNo(
              frame,
              "The chosen file " + chosenFile.getName()
                  + " already exists.\nDo you want to overwrite?",
              "File already exists", "Yes", "No")) {
        return;
      }
      lastSavedFile = chosenFile;
      saveToFile(lastSavedFile);
    }
  }

  /**
   * Called when the user presses the "Cancel" button or closes the window.
   */
  private void cancel()
  {
    rollback();
    ok();
  }

  /**
   * Called when the user presses the "OK" button.
   */
  private void ok()
  {
    unregisterListeners();
    close();
  }
}
