package performancemonster.test.analyzer;

import java.util.Iterator;

import performancemonster.analyzer.LogAnalyzer;
import performancemonster.graph.GraphElement;
import performancemonster.graph.GraphModel;
import performancemonster.loghandler.LogMessage;
import junit.framework.TestCase;

public class LogAnalyzerTest extends TestCase
{
  private GraphModel model;
  private LogAnalyzer analyzer;
  
  @Override
  protected void setUp() throws Exception
  {
    model = new GraphModel();
    analyzer = new LogAnalyzer(model);
  }
  
  public void testSinglePair() throws Exception
  {
    analyzer.receiveLogMessage(createLogMessage("category", "BEGIN message",
        "stack trace", 100, 1));
    analyzer.receiveLogMessage(createLogMessage("category", "END message",
        "stack trace", 200, 1));
    assertEquals("Number of elements incorrect", 1, model.getElements().size());
    GraphElement element = model.getElements().iterator().next();
    assertEquals("Category of element is incorrect", "category", element
        .getCategory());
    assertEquals("Message of element is incorrect", "message", element
        .getMessage());
    assertEquals("Stack trace of element is incorrect", "stack trace", element
        .getStackTrace());
    assertEquals("Beginning is incorrect", 100, element.getBeginning());
    assertEquals("End is incorrect", 200, element.getEnd());
  }

  public void testEndInBeginnings() throws Exception
  {
    // First, put in an END message, which should just be avoided
    analyzer.receiveLogMessage(createLogMessage("category", "END message",
        "stack trace", 50, 1));
    // Then the testSinglePair must work as before
    testSinglePair();
  }
  
  public void testOverlappingPairs() throws Exception
  {
    analyzer.receiveLogMessage(createLogMessage("category1", "BEGIN message1",
        "stack trace1", 100, 1));
    analyzer.receiveLogMessage(createLogMessage("category2", "BEGIN message2",
        "stack trace2", 200, 1));
    analyzer.receiveLogMessage(createLogMessage("category1", "END message1",
        "stack trace1", 300, 1));
    analyzer.receiveLogMessage(createLogMessage("category2", "END message2",
        "stack trace2", 400, 1));
    assertEquals("Number of elements incorrect", 2, model.getElements().size());
    Iterator<GraphElement> it = model.getElements().iterator();
    
    // Verify 1st element
    GraphElement element = it.next();
    assertEquals("Category of element 1 is incorrect", "category1", element
        .getCategory());
    assertEquals("Message of element 1 is incorrect", "message1", element
        .getMessage());
    assertEquals("Stack trace of element 1 is incorrect", "stack trace1",
        element.getStackTrace());
    assertEquals("Beginning of element 1 is incorrect", 100, element
        .getBeginning());
    assertEquals("End of element 1 is incorrect", 300, element.getEnd());

    // Verify 2nd element
    element = it.next();
    assertEquals("Category of element 2 is incorrect", "category2", element
        .getCategory());
    assertEquals("Message of element 2 is incorrect", "message2", element
        .getMessage());
    assertEquals("Stack trace of element 2 is incorrect", "stack trace2",
        element.getStackTrace());
    assertEquals("Beginning of element 2 is incorrect", 200, element
        .getBeginning());
    assertEquals("End of element 2 is incorrect", 400, element.getEnd());
  }
  
  public void testInnerPair() throws Exception
  {
    analyzer.receiveLogMessage(createLogMessage("category1", "BEGIN message1",
        "stack trace1", 100, 1));
    analyzer.receiveLogMessage(createLogMessage("category2", "BEGIN message2",
        "stack trace2", 200, 1));
    analyzer.receiveLogMessage(createLogMessage("category2", "END message2",
        "stack trace2", 300, 1));
    analyzer.receiveLogMessage(createLogMessage("category1", "END message1",
        "stack trace1", 400, 1));
    assertEquals("Number of elements incorrect", 2, model.getElements().size());
    Iterator<GraphElement> it = model.getElements().iterator();
    
    // Verify 1st element
    GraphElement element = it.next();
    assertEquals("Category of element 1 is incorrect", "category1", element
        .getCategory());
    assertEquals("Message of element 1 is incorrect", "message1", element
        .getMessage());
    assertEquals("Stack trace of element 1 is incorrect", "stack trace1",
        element.getStackTrace());
    assertEquals("Beginning of element 1 is incorrect", 100, element
        .getBeginning());
    assertEquals("End of element 1 is incorrect", 400, element.getEnd());

    // Verify 2nd element
    element = it.next();
    assertEquals("Category of element 2 is incorrect", "category2", element
        .getCategory());
    assertEquals("Message of element 2 is incorrect", "message2", element
        .getMessage());
    assertEquals("Stack trace of element 2 is incorrect", "stack trace2",
        element.getStackTrace());
    assertEquals("Beginning of element 2 is incorrect", 200, element
        .getBeginning());
    assertEquals("End of element 2 is incorrect", 300, element.getEnd());
  }
  
  public void testOverlappingEqualPairsInDifferentThreads() throws Exception
  {
    analyzer.receiveLogMessage(createLogMessage("category1", "BEGIN message1",
        "stack trace1", 100, 1));
    analyzer.receiveLogMessage(createLogMessage("category1", "BEGIN message1",
        "stack trace2", 200, 2));
    analyzer.receiveLogMessage(createLogMessage("category1", "END message1",
        "stack trace1", 300, 1));
    analyzer.receiveLogMessage(createLogMessage("category1", "END message1",
        "stack trace2", 400, 2));
    assertEquals("Number of elements incorrect", 2, model.getElements().size());
    Iterator<GraphElement> it = model.getElements().iterator();
    
    // Verify 1st element
    GraphElement element = it.next();
    assertEquals("Category of element 1 is incorrect", "category1", element
        .getCategory());
    assertEquals("Message of element 1 is incorrect", "message1", element
        .getMessage());
    assertEquals("Stack trace of element 1 is incorrect", "stack trace1",
        element.getStackTrace());
    assertEquals("Beginning of element 1 is incorrect", 100, element
        .getBeginning());
    assertEquals("End of element 1 is incorrect", 300, element.getEnd());

    // Verify 2nd element
    element = it.next();
    assertEquals("Category of element 2 is incorrect", "category1", element
        .getCategory());
    assertEquals("Message of element 2 is incorrect", "message1", element
        .getMessage());
    assertEquals("Stack trace of element 2 is incorrect", "stack trace2",
        element.getStackTrace());
    assertEquals("Beginning of element 2 is incorrect", 200, element
        .getBeginning());
    assertEquals("End of element 2 is incorrect", 400, element.getEnd());
  }
  
  private LogMessage createLogMessage(final String category,
      final String message, final String stackTrace, final long timestamp,
      final long threadID)
  {
    return new LogMessage(category, message)
    {
      @Override
      public String getStackTrace()
      {
        return stackTrace;
      }

      @Override
      public long getTimestamp()
      {
        return timestamp;
      }

      @Override
      public long getThreadID()
      {
        return threadID;
      }
    };
  }
}
