package performancemonster.test.graph.coloring;

import java.awt.Color;

import static org.easymock.EasyMock.*;

import performancemonster.graph.coloring.ColorChangeListener;
import performancemonster.graph.coloring.ColorRule;
import performancemonster.graph.coloring.Colorer;
import performancemonster.helpers.MockFactory;
import junit.framework.TestCase;

public class ColorerTest extends TestCase
{
  public void testEmptyColorer()
  {
    Colorer colorer = new Colorer();

    assertEquals("Default bar color is red", Color.RED, colorer
        .chooseBarColor(MockFactory.createGraphElement("Category", "Message",
            "Stack trace", 0, 100)));
    assertEquals("Default border color is black", Color.BLACK, colorer
        .chooseBorderColor(MockFactory.createGraphElement("Category",
            "Message", "Stack trace", 0, 100)));
  }

  public void testColorerWithOneRule()
  {
    Colorer colorer = new Colorer();
    ColorRule rule = new ColorRule("Rule");
    Color ruleBarColor = Color.WHITE;
    Color ruleBorderColor = Color.GRAY;
    rule.setBarColor(ruleBarColor);
    rule.setBorderColor(ruleBorderColor);
    rule.setCategory("Correct category");
    colorer.appendColorRule(rule);

    assertEquals(
        "Default bar color should be chosen when element does not match",
        Color.RED, colorer.chooseBarColor(MockFactory.createGraphElement(
            "Category", "Message", "Stack trace", 0, 100)));
    assertEquals(
        "Default border color should be chosen when element does not match",
        Color.BLACK, colorer.chooseBorderColor(MockFactory.createGraphElement(
            "Category", "Message", "Stack trace", 0, 100)));

    assertEquals("The rule's bar color should be chosen when element matches",
        ruleBarColor, colorer.chooseBarColor(MockFactory.createGraphElement(
            "Correct category", "Message", "Stack trace", 0, 100)));
    assertEquals(
        "The rule's border color should be chosen when element matches",
        ruleBorderColor, colorer.chooseBorderColor(MockFactory
            .createGraphElement("Correct category", "Message", "Stack trace",
                0, 100)));
  }

  public void testMoveUp()
  {
    Colorer colorer = new Colorer();
    ColorRule rule1 = new ColorRule("Rule1");
    ColorRule rule2 = new ColorRule("Rule2");
    colorer.appendColorRule(rule1);
    colorer.appendColorRule(rule2);

    assertSame("Rule 1 must be first", rule1, colorer.getColorRule(0));
    assertSame("Rule 2 must be last", rule2, colorer.getColorRule(1));

    colorer.moveUp(rule2);
    assertSame("Rule 2 must be first", rule2, colorer.getColorRule(0));
    assertSame("Rule 1 must be last", rule1, colorer.getColorRule(1));

    // Move up an extra time, to check that things don't blow up
    colorer.moveUp(rule2);
    assertSame("Rule 2 must be first", rule2, colorer.getColorRule(0));
    assertSame("Rule 1 must be last", rule1, colorer.getColorRule(1));

    // Move up a rule that is not in the rule list
    try {
      ColorRule ruleNotInColorer = new ColorRule("Confusing rule"); 
      colorer.moveUp(ruleNotInColorer);
      fail("Did not fail when moveing up a non-existent rule");
    } catch (IllegalArgumentException e) {
      // Intentionally left blank
    }
  }

  public void testMoveDown()
  {
    Colorer colorer = new Colorer();
    ColorRule rule1 = new ColorRule("Rule1");
    ColorRule rule2 = new ColorRule("Rule2");
    colorer.appendColorRule(rule1);
    colorer.appendColorRule(rule2);

    assertSame("Rule 1 must be first", rule1, colorer.getColorRule(0));
    assertSame("Rule 2 must be last", rule2, colorer.getColorRule(1));

    colorer.moveDown(rule1);
    assertSame("Rule 2 must be first", rule2, colorer.getColorRule(0));
    assertSame("Rule 1 must be last", rule1, colorer.getColorRule(1));

    // Move down an extra time, to check that things don't blow up
    colorer.moveDown(rule1);
    assertSame("Rule 2 must be first", rule2, colorer.getColorRule(0));
    assertSame("Rule 1 must be last", rule1, colorer.getColorRule(1));

    // Move down a rule that is not in the rule list
    try {
      ColorRule ruleNotInColorer = new ColorRule("Confusing rule"); 
      colorer.moveDown(ruleNotInColorer);
      fail("Did not fail when moveing down a non-existent rule");
    } catch (IllegalArgumentException e) {
      // Intentionally left blank
    }
  }

  public void testCopy()
  {
    Colorer colorer = new Colorer();
    colorer.setBackgroundColor(Color.BLUE);
    colorer.setDefaultBarColor(Color.YELLOW);
    colorer.setDefaultBorderColor(Color.CYAN);
    ColorRule rule = new ColorRule("Rule");
    colorer.appendColorRule(rule);
    Colorer colorerCopy = colorer.copy();

    assertCopyCorrect(colorer, colorerCopy);

    colorerCopy.setBackgroundColor(Color.BLACK);
    colorerCopy.getColorRule(0).setName("Other name");
    assertFalse("Original and copy of background must not match", colorer
        .getBackgroundColor().equals(colorerCopy.getBackgroundColor()));
    assertFalse("Original and copy of color rule name must not match", colorer
        .getColorRule(0).getName()
        .equals(colorerCopy.getColorRule(0).getName()));
  }

  public void testReplace()
  {
    Colorer colorer = new Colorer();
    colorer.setBackgroundColor(Color.BLUE);
    colorer.setDefaultBarColor(Color.YELLOW);
    colorer.setDefaultBorderColor(Color.CYAN);
    ColorRule rule = new ColorRule("Rule");
    colorer.appendColorRule(rule);

    Colorer colorerCopy = colorer.copy();
    colorerCopy.setBackgroundColor(Color.BLACK);
    colorerCopy.getColorRule(0).setName("Other name");
    colorer.replaceWith(colorerCopy);

    assertCopyCorrect(colorer, colorerCopy);
    assertEquals("Resulting background color must be from the copy",
        Color.BLACK, colorer.getBackgroundColor());
    assertEquals("Resulting rule name must be from the copy", "Other name",
        colorerCopy.getColorRule(0).getName());
  }

  public void testNotificationSetBackgroundColor()
  {
    Colorer colorer = new Colorer();

    ColorChangeListener listener = createMock(ColorChangeListener.class);
    colorer.addChangeListener(listener);

    listener.coloringChanged();
    replay(listener);
    colorer.setBackgroundColor(Color.RED);
    verify(listener);
  }

  public void testNotificationSetDefaultBarColor()
  {
    Colorer colorer = new Colorer();

    ColorChangeListener listener = createMock(ColorChangeListener.class);
    colorer.addChangeListener(listener);

    listener.coloringChanged();
    replay(listener);
    colorer.setDefaultBarColor(Color.RED);
    verify(listener);
  }

  public void testNotificationSetDefaultBorderColor()
  {
    Colorer colorer = new Colorer();

    ColorChangeListener listener = createMock(ColorChangeListener.class);
    colorer.addChangeListener(listener);

    listener.coloringChanged();
    replay(listener);
    colorer.setDefaultBorderColor(Color.RED);
    verify(listener);
  }

  public void testNotificationChangeRule()
  {
    Colorer colorer = new Colorer();
    ColorRule rule = new ColorRule("Rule");
    colorer.appendColorRule(rule);

    ColorChangeListener listener = createMock(ColorChangeListener.class);
    colorer.addChangeListener(listener);

    listener.coloringChanged();
    replay(listener);
    rule.setName("Dummy rule name");
    verify(listener);
  }

  public void testNotificationNotOnCopy()
  {
    Colorer colorer = new Colorer();
    colorer.appendColorRule(new ColorRule("Rule"));
    Colorer colorerCopy = colorer.copy();

    ColorChangeListener listener = createMock(ColorChangeListener.class);
    ColorChangeListener listenerCopy = createMock(ColorChangeListener.class);
    colorer.addChangeListener(listener);
    colorerCopy.addChangeListener(listenerCopy);

    changeColorerAndColorerCopy(colorer, listener, colorerCopy, listenerCopy);
  }

  public void testNotificationSeparatedWhenCopied()
  {
    Colorer colorer = new Colorer();
    colorer.appendColorRule(new ColorRule("Rule"));
    ColorChangeListener listener = createMock(ColorChangeListener.class);
    colorer.addChangeListener(listener);

    Colorer colorerCopy = colorer.copy();
    ColorChangeListener listenerCopy = createMock(ColorChangeListener.class);
    colorerCopy.addChangeListener(listenerCopy);

    changeColorerAndColorerCopy(colorer, listener, colorerCopy, listenerCopy);
  }

  public void testNotificationSeparatedWhenCopiedAndReplaced()
  {
    Colorer colorer = new Colorer();
    colorer.appendColorRule(new ColorRule("Rule"));
    Colorer colorerCopy = colorer.copy();

    ColorChangeListener listener = createMock(ColorChangeListener.class);
    ColorChangeListener listenerCopy = createMock(ColorChangeListener.class);
    colorer.addChangeListener(listener);
    colorerCopy.addChangeListener(listenerCopy);

    listener.coloringChanged();
    replay(listener);
    replay(listenerCopy);
    colorer.replaceWith(colorerCopy);
    reset(listener);
    reset(listenerCopy);

    changeColorerAndColorerCopy(colorer, listener, colorerCopy, listenerCopy);
  }

  private void assertCopyCorrect(Colorer colorer, Colorer colorerCopy)
  {
    assertEquals("Original and copy of background must be equal", colorer
        .getBackgroundColor(), colorerCopy.getBackgroundColor());
    assertEquals("Original and copy of default bar must be equal", colorer
        .getDefaultBarColor(), colorerCopy.getDefaultBarColor());
    assertEquals("Original and copy of default border must be equal", colorer
        .getDefaultBorderColor(), colorerCopy.getDefaultBorderColor());
    assertNotSame("Original and copy of rule must be different", colorer
        .getColorRule(0), colorerCopy.getColorRule(0));
    assertEquals("Original and copy of color rule name must be equal", colorer
        .getColorRule(0).getName(), colorerCopy.getColorRule(0).getName());
  }

  private void changeColorerAndColorerCopy(Colorer colorer,
      ColorChangeListener listener, Colorer colorerCopy,
      ColorChangeListener listenerCopy)
  {
    // Change "colorer" directly
    listener.coloringChanged();
    replay(listener);
    replay(listenerCopy);
    colorer.setBackgroundColor(Color.RED);
    verify(listener);
    verify(listenerCopy);

    reset(listener);
    reset(listenerCopy);

    // Change "colorer" through its color rule
    listener.coloringChanged();
    replay(listener);
    replay(listenerCopy);
    colorer.getColorRule(0).setName("Dummy name");
    verify(listener);
    verify(listenerCopy);

    reset(listener);
    reset(listenerCopy);

    // Change "colorerCopy" directly
    listenerCopy.coloringChanged();
    replay(listener);
    replay(listenerCopy);
    colorerCopy.setBackgroundColor(Color.RED);
    verify(listener);
    verify(listenerCopy);

    reset(listener);
    reset(listenerCopy);

    // Change "colorerCopy" through its color rule
    listenerCopy.coloringChanged();
    replay(listener);
    replay(listenerCopy);
    colorerCopy.getColorRule(0).setName("Dummy name");
    verify(listener);
    verify(listenerCopy);
  }
}
